@@ -1,6 +1,6 @@
 twsBAG <-
 function(...) {
-  combolegs <- as.list(...)
+  combolegs <- list(...)
   twsContract(0,"USD","BAG","SMART","","","0.0","USD","","","",NULL,combolegs,"0","","")
 }
 
